@@ -252,12 +252,17 @@ create_plug (BonoboControl *control)
 	
 	plug = bonobo_plug_new (0);
 	gtk_widget_show (plug);
+
+	g_object_ref (G_OBJECT (plug));
+	gtk_object_sink (GTK_OBJECT (plug));
 	
 	bonobo_control_set_plug (control, BONOBO_PLUG (plug));
 
 	if (control->priv->widget)
 		gtk_container_add (GTK_CONTAINER (plug),
 				   control->priv->widget);
+
+	g_object_unref (G_OBJECT (plug));
 }
 
 static CORBA_char *
@@ -584,8 +589,10 @@ bonobo_control_destroy (BonoboObject *object)
 	bonobo_control_set_properties      (control, CORBA_OBJECT_NIL, NULL);
 	bonobo_control_set_ui_component    (control, NULL);
 
-	if (control->priv->widget)
+	if (control->priv->widget) {
+		gtk_widget_destroy (GTK_WIDGET (control->priv->widget));
 		g_object_unref (control->priv->widget);
+	}
 	control->priv->widget = NULL;
 
 	control->priv->popup_ui_container = bonobo_object_unref (
@@ -799,6 +806,9 @@ bonobo_control_get_remote_ui_container (BonoboControl     *control,
 
 	bonobo_object_check_env (BONOBO_OBJECT (control), control->priv->frame, ev);
 
+	if (BONOBO_EX (ev))
+		ui_container = CORBA_OBJECT_NIL;
+
 	if (!opt_ev)
 		CORBA_exception_free (&tmp_ev);
 
@@ -1029,7 +1039,7 @@ window_transient_destroy_gdk_cb (GtkWidget *widget)
 	GdkWindow *win;
 	
 	if ((win = g_object_get_data (G_OBJECT (widget), "transient")))
-		gdk_window_unref (win);
+		g_object_unref (win);
 }
 
 static void       
@@ -1040,7 +1050,7 @@ window_set_transient_for_gdk (GtkWindow *window,
 	g_return_if_fail (g_object_get_data (
 		G_OBJECT (window), "transient") == NULL);
 
-	gdk_window_ref (parent);
+	g_object_ref (parent);
 
 	g_object_set_data (G_OBJECT (window), "transient", parent);
 
@@ -1105,7 +1115,7 @@ bonobo_control_set_transient_for (BonoboControl     *control,
 	}
 
 	id = Bonobo_ControlFrame_getToplevelId (frame, ev);
-	g_return_if_fail (id != NULL);
+	g_return_if_fail (!BONOBO_EX (ev) && id != NULL);
 
 	x11_id = bonobo_control_x11_from_window_id (id);
 
@@ -1160,18 +1170,24 @@ bonobo_control_set_plug (BonoboControl *control,
 	if ((BonoboPlug *) control->priv->plug == plug)
 		return;
 
+	dprintf ("bonobo_control_set_plug (%p, %p) [%p]\n",
+		 control, plug, control->priv->plug);
+
 	old_plug = (BonoboPlug *) control->priv->plug;
 
-	if (plug) {
+	if (plug)
 		control->priv->plug = g_object_ref (plug);
-		bonobo_plug_set_control (plug, control);
-	} else
+	else
 		control->priv->plug = NULL;
 
 	if (old_plug) {
 		bonobo_plug_set_control (old_plug, NULL);
+		gtk_widget_destroy (GTK_WIDGET (old_plug));
 		g_object_unref (old_plug);
 	}
+
+	if (plug)
+		bonobo_plug_set_control (plug, control);
 }
 
 BonoboPlug *
@@ -1201,15 +1217,24 @@ bonobo_control_get_popup_ui_container (BonoboControl *control)
 	g_return_val_if_fail (BONOBO_IS_CONTROL (control), NULL);
 
 	if (!control->priv->popup_ui_container) {
-		control->priv->popup_ui_engine = bonobo_ui_engine_new (
-			G_OBJECT (control));
+		BonoboUIEngine *ui_engine;
+		BonoboUISync   *ui_sync;
 
-		control->priv->popup_ui_sync = bonobo_ui_sync_menu_new (
-			control->priv->popup_ui_engine, NULL, NULL, NULL);
+		ui_engine = bonobo_ui_engine_new (G_OBJECT (control));
 
-		bonobo_ui_engine_add_sync (
-			control->priv->popup_ui_engine,
-			control->priv->popup_ui_sync);
+		ui_sync = bonobo_ui_sync_menu_new (ui_engine, NULL, NULL, NULL);
+
+		bonobo_ui_engine_add_sync (ui_engine, ui_sync);
+
+		/* re-entrancy guard */
+		if (control->priv->popup_ui_container) {
+			g_object_unref (ui_engine);
+
+			return control->priv->popup_ui_container;
+		}
+
+		control->priv->popup_ui_engine = ui_engine;
+		control->priv->popup_ui_sync   = ui_sync;
 
 		control->priv->popup_ui_container = bonobo_ui_container_new ();
 		bonobo_ui_container_set_engine (
