@@ -87,18 +87,23 @@ bonobo_plug_set_control (BonoboPlug    *plug,
 	if (plug->control == control)
 		return;
 
+	dprintf ("bonobo_plug_set_control (%p, %p) [%p]\n",
+		 plug, control, plug->control);
+
 	old_control = plug->control;
 
-	if (control) {
+	if (control)
 		plug->control = g_object_ref (control);
-		bonobo_control_set_plug (control, plug);
-	} else
+	else
 		plug->control = NULL;
 
 	if (old_control) {
 		bonobo_control_set_plug (old_control, NULL);
 		g_object_unref (old_control);
 	}
+
+	if (control)
+		bonobo_control_set_plug (control, plug);
 }
 
 static gboolean
@@ -280,9 +285,16 @@ bonobo_plug_button_event (GtkWidget      *widget,
 	if (!BONOBO_PLUG (widget)->priv->forward_events || !GTK_WIDGET_TOPLEVEL (widget))
 		return FALSE;
 
-	if (event->type == GDK_BUTTON_PRESS)
+	if (event->type == GDK_BUTTON_PRESS) {
 		xevent.xbutton.type = ButtonPress;
-	else
+
+		/* X does an automatic pointer grab on button press
+		 * if we have both button press and release events
+		 * selected.
+		 * We don't want to hog the pointer on our parent.
+		 */
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	} else
 		xevent.xbutton.type = ButtonRelease;
     
 	xevent.xbutton.display     = GDK_WINDOW_XDISPLAY (widget->window);
