@@ -67,22 +67,7 @@ enum {
 	LAST_SIGNAL
 };
 
-guint signals [LAST_SIGNAL] = { 0 };
-
-struct _BonoboUIEnginePrivate {
-	GObject      *view;
-
-	BonoboUIXml  *tree;
-	int           frozen;
-	GSList       *syncs;
-	GSList       *state_updates;
-	GSList       *components;
-
-	BonoboUIContainer    *container;
-	BonoboUIEngineConfig *config;
-
-	GHashTable   *cmd_to_node;
-};
+static guint signals [LAST_SIGNAL] = { 0 };
 
 /*
  *  Mapping from nodes to their synchronization
@@ -1426,18 +1411,19 @@ bonobo_ui_engine_set_ui_container (BonoboUIEngine    *engine,
 
 	old_container = engine->priv->container;
 
-	if (ui_container) {
+	if (ui_container)
 		engine->priv->container = BONOBO_UI_CONTAINER (
 			bonobo_object_ref (BONOBO_OBJECT (ui_container)));
-
-		bonobo_ui_container_set_engine (ui_container, engine);
-	} else
+	else
 		engine->priv->container = NULL;
 
 	if (old_container) {
 		bonobo_ui_container_set_engine (old_container, NULL);
 		bonobo_object_unref (BONOBO_OBJECT (old_container));
 	}
+
+	if (ui_container)
+		bonobo_ui_container_set_engine (ui_container, engine);
 }
 
 /**
@@ -1983,19 +1969,19 @@ bonobo_ui_engine_construct (BonoboUIEngine *engine,
 
 	build_skeleton (priv->tree);
 
-	g_signal_connect (G_OBJECT (priv->tree), "override",
+	g_signal_connect (priv->tree, "override",
 			  (GtkSignalFunc) override_fn, engine);
 
-	g_signal_connect (G_OBJECT (priv->tree), "replace_override",
+	g_signal_connect (priv->tree, "replace_override",
 			  (GtkSignalFunc) replace_override_fn, engine);
 
-	g_signal_connect (G_OBJECT (priv->tree), "reinstate",
+	g_signal_connect (priv->tree, "reinstate",
 			  (GtkSignalFunc) reinstate_fn, engine);
 
-	g_signal_connect (G_OBJECT (priv->tree), "rename",
+	g_signal_connect (priv->tree, "rename",
 			  (GtkSignalFunc) rename_fn, engine);
 
-	g_signal_connect (G_OBJECT (priv->tree), "remove",
+	g_signal_connect (priv->tree, "remove",
 			  (GtkSignalFunc) remove_fn, engine);
 
 	/* Add the engine to the configuration notify list */
